@@ -27,7 +27,6 @@
 #include "quota.h"
 #include "fs.h"
 #include "dist.h"
-#include "cpt.h"
 #include "meminfo.h"
 #include "veth.h"
 
@@ -70,6 +69,15 @@ typedef struct name_param {
 	char *name;
 } name_param;
 
+typedef struct cpt_param {
+	char *dumpdir;
+	char *dumpfile;
+	unsigned int ctx;
+	unsigned int cpu_flags;
+	int cmd;
+	int rst_fd;
+} cpt_param;
+
 /** Data structure for CT resources.
  */
 typedef struct vps_res {
@@ -95,6 +103,11 @@ enum {
 	APPCONF_MAP_NAME = 0x01,
 };
 
+enum {
+	VE_LAYOUT_SIMFS = 1,
+	VE_LAYOUT_PLOOP = 2,
+};
+
 typedef struct {
 	int save;
 	int save_force;
@@ -107,6 +120,8 @@ typedef struct {
 	char *config;
 	char *origin_sample;
 	char *lockdir;
+	int layout; /* VE_LAYOUT_* */
+	int mode; /* PLOOP_*_MODE */
 	char *apply_cfg;
 	int apply_cfg_map;
 	int reset_ub;
@@ -119,11 +134,18 @@ struct log_s {
 	int *verbose;
 };
 
+struct vzctl_snapshot_param {
+	char *guid;
+	char *name;
+	char *desc;
+};
+
 struct vps_param {
 	struct log_s log;
 	vps_res res;
 	vps_res del_res;
 	vps_opt opt;
+	struct vzctl_snapshot_param snap;
 	struct mod_action *mod;
 	struct vps_param *g_param;
 };
@@ -146,4 +168,9 @@ int vps_setup_res(vps_handler *h, envid_t veid, dist_actions *actions,
 int setup_resource_management(vps_handler *h, envid_t veid, vps_res *res);
 int need_configure(vps_res *res);
 
+/* Second-level quota */
+struct setup_env_quota_param;
+int fill_2quota_param(struct setup_env_quota_param *p,
+		const char *ve_private, const char *ve_root);
+
 #endif
