@@ -138,6 +138,11 @@
 #define PARAM_PCI_DEL		366
 #define PARAM_RAM		367
 #define PARAM_SWAP		368
+#define PARAM_VE_LAYOUT		369
+
+#define PARAM_SNAPSHOT_GUID	370
+#define PARAM_SNAPSHOT_NAME	371
+#define PARAM_SNAPSHOT_DESC	372
 
 #define PARAM_LINE		"e:p:f:t:i:l:k:a:b:n:x:h"
 #endif
