@@ -148,4 +148,17 @@
 #define VZ_CONFIG_SAVE_ERROR		139
 
 #define VZ_SET_OSRELEASE		144
+#define VZ_SET_IO			148
+
+/* ploop */
+#define VZ_NO_IMAGE_FILE		150
+#define VZCTL_E_CREATE_IMAGE		151
+#define VZCTL_E_MOUNT_IMAGE		152
+#define VZCTL_E_UMOUNT_IMAGE		153
+#define VZCTL_E_RESIZE_IMAGE		154
+#define VZCTL_E_CONVERT_IMAGE		155
+#define VZCTL_E_CREATE_SNAPSHOT		156
+#define VZCTL_E_MERGE_SNAPSHOT		157
+#define VZCTL_E_DELETE_SNAPSHOT		158
+#define VZCTL_E_SWITCH_SNAPSHOT		159
 #endif /* _VZ_ERROR_H_ */
