@@ -38,6 +38,7 @@
 #include "modules.h"
 #include "create.h"
 #include "destroy.h"
+#include "image.h"
 
 #define VPS_CREATE	LIB_SCRIPTS_DIR "vps-create"
 #define VPS_DOWNLOAD	LIB_SCRIPTS_DIR "vps-download"
@@ -73,15 +74,23 @@ static char *get_ostemplate_name(char *ostmpl)
 static int download_template(char *tmpl)
 {
 	char *arg[3];
+	char *env[] = { NULL, NULL };
+	char *val;
+	char buf[64];
 
 	arg[0] = VPS_DOWNLOAD;
 	arg[1] = tmpl;
 	arg[2] = NULL;
-	return run_script(VPS_DOWNLOAD, arg, NULL, 0);
+	val = getenv("HOME");
+	if (val != NULL) {
+		snprintf(buf, sizeof(buf), "HOME=%s", val);
+		env[0] = buf;
+	}
+	return run_script(VPS_DOWNLOAD, arg, env, 0);
 }
 
 static int fs_create(envid_t veid, fs_param *fs, tmpl_param *tmpl,
-	dq_param *dq)
+	dq_param *dq, int layout, int ploop_mode)
 {
 	char tarball[PATH_LEN];
 	char tmp_dir[PATH_LEN];
@@ -91,9 +100,15 @@ static int fs_create(envid_t veid, fs_param *fs, tmpl_param *tmpl,
 	char *env[4];
 	int quota = 0;
 	int i;
+	char *dst;
 	const char *ext[] = { "", ".gz", ".bz2", ".xz", NULL };
 	const char *errmsg_ext = "[.gz|.bz2|.xz]";
+	int ploop = (layout == VE_LAYOUT_PLOOP);
 
+	if (ploop && (!dq->diskspace || dq->diskspace[1] <= 0)) {
+		logger(-1, 0, "Error: diskspace not set (required for ploop)");
+		return VZ_DISKSPACE_NOT_SET;
+	}
 find:
 	for (i = 0; ext[i] != NULL; i++) {
 		snprintf(tarball, sizeof(tarball), "%s/cache/%s.tar%s",
@@ -126,7 +141,34 @@ find:
 		ret = VZ_FS_NEW_VE_PRVT;
 		goto err;
 	}
-	if (dq != NULL &&
+	dst = tmp_dir;
+	if (ploop) {
+		/* Create and mount ploop image */
+		struct vzctl_create_image_param param = {};
+		struct vzctl_mount_param mount_param = {};
+
+		if (ploop_mode < 0)
+			ploop_mode = PLOOP_EXPANDED_MODE;
+		param.mode = ploop_mode;
+		param.size = dq->diskspace[1]; // limit
+		ret = vzctl_create_image(tmp_dir, &param);
+		if (ret)
+			goto err;
+
+		if (make_dir(fs->root, 1)) {
+			logger(-1, 0, "Can't create mount point %s", fs->root);
+			ret = VZ_FS_MPOINTCREATE;
+			goto err;
+		}
+		mount_param.target = fs->root;
+		ret = vzctl_mount_image(tmp_dir, &mount_param);
+		if (ret)
+			goto err;
+
+		dst = fs->root;
+	}
+	if (!ploop &&
+		dq != NULL &&
 		dq->enable == YES &&
 		dq->diskspace != NULL &&
 		dq->diskinodes != NULL)
@@ -142,12 +184,15 @@ find:
 	arg[1] = NULL;
 	snprintf(buf, sizeof(buf), "PRIVATE_TEMPLATE=%s", tarball);
 	env[0] = strdup(buf);
-	snprintf(buf, sizeof(buf), "VE_PRVT=%s", tmp_dir);
+	snprintf(buf, sizeof(buf), "VE_PRVT=%s", dst);
 	env[1] = strdup(buf);
 	env[2] = strdup(ENV_PATH);
 	env[3] = NULL;
+	logger(0, 0, "Creating container private area (%s)", tmpl->ostmpl);
 	ret = run_script(VPS_CREATE, arg, env, 0);
 	free_arg(env);
+	if (ploop)
+		vzctl_umount_image(tmp_dir);
 	if (ret)
 		goto err;
 	if (quota) {
@@ -169,8 +214,8 @@ err:
 		quota_off(veid, 0);
 		quota_ctl(veid, QUOTA_DROP);
 	}
-	rmdir(fs->private);
-	rmdir(tmp_dir);
+	rmdir(fs->private); /* remove VE_PRIVATE if empty */
+	del_dir(tmp_dir);
 
 	return ret;
 }
@@ -197,7 +242,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 		snprintf(src, sizeof(src),  VPS_CONF_DIR "ve-%s.conf-sample",
 			cmd_p->opt.config);
 		if (!stat_file(src)) {
-			logger(-1, 0, "File %s is not found", src);
+			logger(-1, errno, "Sample config %s not found", src);
 			ret = VZ_CP_CONFIG;
 			goto err;
 		}
@@ -214,7 +259,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 			vps_p->opt.config);
 		/* Bail out if CONFIGFILE is wrong in /etc/vz/vz.conf */
 		if (!stat_file(src)) {
-			logger(-1, errno, "File %s not found", src);
+			logger(-1, errno, "Sample config %s not found", src);
 			logger(-1, 0, "Fix the value of CONFIGFILE in "
 					GLOBAL_CFG);
 			ret = VZ_CP_CONFIG;
@@ -255,21 +300,26 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 		ret = VZ_VE_TMPL_NOTSET;
 		goto err_cfg;
 	}
-	else if (check_var(fs->private, "VE_PRIVATE is not set"))
+	if (check_var(fs->private, "VE_PRIVATE is not set"))
 	{
 		ret = VZ_VE_PRIVATE_NOTSET;
 		goto err_cfg;
 	}
-	else if (check_var(fs->root, "VE_ROOT is not set"))
+	if (check_var(fs->root, "VE_ROOT is not set"))
 	{
 		ret = VZ_VE_ROOT_NOTSET;
 		goto err_cfg;
 	}
-	else if (stat_file(fs->private)) {
+	if (stat_file(fs->private)) {
 		logger(-1, 0, "Private area already exists in %s", fs->private);
 		ret = VZ_FS_PRVT_AREA_EXIST;
 		goto err_cfg;
 	}
+	if (vps_p->opt.layout == VE_LAYOUT_PLOOP && !is_ploop_supported()) {
+		logger(-1, 0, "No ploop support in the kernel");
+		ret = VZ_BAD_KERNEL;
+		goto err_cfg;
+	}
 
 	if (cmd_p->res.name.name) {
 		if ((ret = set_name(veid, cmd_p->res.name.name,
@@ -301,10 +351,10 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 				tmpl->ostmpl = full_ostmpl;
 			}
 		}
-		logger(0, 0, "Creating container private area (%s)",
-				tmpl->ostmpl);
-		if ((ret = fs_create(veid, fs, tmpl, &vps_p->res.dq)))
-			goto err_private;
+		if ((ret = fs_create(veid, fs, tmpl, &vps_p->res.dq,
+						vps_p->opt.layout,
+						vps_p->opt.mode)))
+			goto err_root;
 	}
 
 	if ((ret = vps_postcreate(veid, &vps_p->res)))
@@ -385,7 +435,7 @@ static int vps_postcreate(envid_t veid, vps_res *res)
 	env[2] = NULL;
 	logger(0, 0, "Performing postcreate actions");
 	ret = run_script(actions.post_create, arg, env, 0);
-	fsumount(veid, res->fs.root);
+	fsumount(veid, &res->fs);
 err:
 	free_dist_actions(&actions);
 	return ret;
