@@ -45,6 +45,7 @@
 #include "readelf.h"
 #include "vzsyscalls.h"
 #include "cpt.h"
+#include "image.h"
 
 #define ENVRETRY	3
 
@@ -78,7 +79,7 @@ static int set_personality32()
 }
 #endif
 
-int vz_env_create_data_ioctl(vps_handler *h,
+static int vz_env_create_data_ioctl(vps_handler *h,
 	struct vzctl_env_create_data *data)
 {
 	int errcode;
@@ -341,7 +342,7 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 		create_param.feature_mask |= VE_FEATURE_SYSFS;
 		create_param.known_features |= VE_FEATURE_SYSFS;
 	}
-	logger(3, 0, "Set features mask %016Lx/%016Lx",
+	logger(3, 0, "Set features mask %016llx/%016llx",
 			create_param.feature_mask,
 			create_param.known_features);
 
@@ -413,8 +414,6 @@ try:
 		mount("sunrpc", "/var/lib/nfs/rpc_pipefs", "rpc_pipefs", 0, 0);
 	}
 	configure_sysctl();
-	if (res->dq.ugidlimit != NULL)
-		mk_quota_link();
 
 	/* Close status descriptor to report that
 	 * environment is created.
@@ -624,7 +623,8 @@ static int fix_ve_systemd(const char *root)
 
 	return 0;
 err:
-	logger(-1, errno, "Failed to set upstart as init");
+	logger(-1, errno, "Failed to set upstart as init. "
+			"Please install upstart into this CT.");
 	return -1;
 }
 
@@ -685,6 +685,7 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	struct sigaction act;
 	vps_res *res = &param->res;
 	dist_actions actions;
+	int ploop;
 
 	memset(&actions, 0, sizeof(actions));
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
@@ -695,6 +696,9 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	}
 	if ((ret = check_ub(&res->ub)))
 		return ret;
+
+	ploop = ve_private_is_ploop(res->fs.private);
+
 	dist_name = get_dist_name(&res->tmpl);
 	ret = read_dist_actions(dist_name, DIST_DIR, &actions);
 	free(dist_name);
@@ -703,14 +707,16 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	logger(0, 0, "Starting container ...");
 	if (vps_is_mounted(res->fs.root)) {
 		/* if CT is mounted -- umount first, to cleanup mount state */
-		vps_umount(h, veid, res->fs.root, skip);
+		vps_umount(h, veid, &res->fs, skip);
 	}
 	if (!vps_is_mounted(res->fs.root)) {
 		/* increase quota to perform setup */
-		quota_inc(&res->dq, 100);
+		if (!ploop)
+			quota_inc(&res->dq, 100);
 		if ((ret = vps_mount(h, veid, &res->fs, &res->dq, skip)))
 			return ret;
-		quota_inc(&res->dq, -100);
+		if (!ploop)
+			quota_inc(&res->dq, -100);
 	}
 	/* Fedora 14/15 hacks */
 	if (fix_ve_devconsole(res->fs.root) != 0)
@@ -819,9 +825,10 @@ err:
 		if (vps_is_run(h, veid))
 			env_stop(h, veid, res->fs.root, M_KILL);
 		/* restore original quota values */
-		vps_set_quota(veid, &res->dq);
+		if (!ploop)
+			vps_set_quota(veid, &res->dq);
 		if (vps_is_mounted(res->fs.root))
-			vps_umount(h, veid, res->fs.root, skip);
+			vps_umount(h, veid, &res->fs, skip);
 	}
 	close(wait_p[0]);
 	close(wait_p[1]);
@@ -988,7 +995,7 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 	/* Cleanup CT IPs */
 	run_net_script(veid, DEL, &param->del_res.net.ip,
 			STATE_STOPPING, param->res.net.skip_arpdetect);
-	ret = vps_umount(h, veid, res->fs.root, skip);
+	ret = vps_umount(h, veid, &res->fs, skip);
 
 end:
 	free_str_param(&param->del_res.net.ip);
