@@ -353,7 +353,7 @@ int vps_exec(vps_handler *h, envid_t veid, const char *root, int exec_mode,
 	return ret;
 }
 
-int _real_execFn(vps_handler *h, envid_t veid, const char *root,
+static int _real_execFn(vps_handler *h, envid_t veid, const char *root,
 		execFn fn, void *data, int flags)
 {
 	int ret, pid;
@@ -487,7 +487,7 @@ int vps_run_script(vps_handler *h, envid_t veid, char *script, vps_param *vps_p)
 		while (retry++ < 10 && vps_is_run(h, veid))
 			usleep(500000);
 		if (!is_mounted)
-			fsumount(veid, root);
+			fsumount(veid, &vps_p->res.fs);
 	}
 	close(rd_p[0]);
 	close(rd_p[1]);
