@@ -67,31 +67,42 @@ static const char *unescapestr(char *src)
 	return src;
 }
 
-char *parse_line(char *str, char *ltoken, int lsz)
+char *parse_line(char *str, char *ltoken, int lsz, char **err)
 {
 	char *sp = str;
-	char *ep, *p;
+	char *ep, *p, *ret;
 	int len;
 
+	*err = NULL;
 	unescapestr(str);
 	while (*sp && isspace(*sp)) sp++;
 	if (!*sp || *sp == '#')
 		return NULL;
 	ep = sp + strlen(sp) - 1;
 	while (isspace(*ep) && ep >= sp) *ep-- = '\0';
-	if (*ep == '"')
-		*ep = 0;
-	if (!(p = strchr(sp, '=')))
+	if (!(p = strchr(sp, '='))) {
+		*err = "'=' not found";
 		return NULL;
+	}
 	len = p - sp;
-	if (len >= lsz)
+	if (len >= lsz) {
+		*err = "too long value";
 		return NULL;
+	}
 	strncpy(ltoken, sp, len);
 	ltoken[len] = 0;
-	if (*(++p) == '"')
-		p++;
+	if (*(++p) != '"')
+		return p;
+	/* Quoted argument requires some additional processing */
+	ret = ++p; /* skip opening quote */
+	/* Find the matching closing quote */
+	if (!(p = strrchr(p, '"'))) {
+		*err="unmatched quotes";
+		return NULL;
+	}
+	*p = '\0';
 
-	return p;
+	return ret;
 }
 
 /*
@@ -111,10 +122,10 @@ int stat_file(const char *file)
 	return 1;
 }
 
-int make_dir(char *path, int full)
+int make_dir_mode(const char *path, int full, int mode)
 {
 	char buf[4096];
-	char *ps, *p;
+	const char *ps, *p;
 	int len;
 
 	if (path == NULL)
@@ -126,7 +137,7 @@ int make_dir(char *path, int full)
 		snprintf(buf, len, "%s", path);
 		ps = p + 1;
 		if (!stat_file(buf)) {
-			if (mkdir(buf, 0755)) {
+			if (mkdir(buf, mode)) {
 				logger(-1, errno, "Can't create directory %s",
 					buf);
 				return 1;
@@ -136,7 +147,7 @@ int make_dir(char *path, int full)
 	if (!full)
 		return 0;
 	if (!stat_file(path)) {
-		if (mkdir(path, 0755)) {
+		if (mkdir(path, mode)) {
 			logger(-1, errno, "Can't create directory %s", path);
 			return 1;
 		}
@@ -144,6 +155,11 @@ int make_dir(char *path, int full)
 	return 0;
 }
 
+int make_dir(const char *path, int full)
+{
+	return make_dir_mode(path, full, 0755);
+}
+
 int parse_int(const char *str, int *val)
 {
 	char *tail;
@@ -184,7 +200,7 @@ int check_var(const void *val, const char *message)
 
 int cp_file(char *dst, char *src)
 {
-	int fd_src, fd_dst, len, ret = 0;
+	int fd_src, fd_dst, ret = 0;
 	struct stat st;
 	char buf[4096];
 
@@ -192,7 +208,6 @@ int cp_file(char *dst, char *src)
 		logger(-1, errno, "Unable to stat %s", src);
 		return -1;
 	}
-	len = st.st_size;
 	if ((fd_src = open(src, O_RDONLY)) < 0) {
 		logger(-1, errno, "Unable to open %s", src);
 		return -1;
