@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2012, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,10 +40,14 @@
 #include "vps_configure.h"
 #include "modules.h"
 #include "io.h"
+#include "image.h"
+#include "cpt.h"
+#include "snapshot.h"
 
 extern struct mod_action g_action;
 extern int do_enter(vps_handler *h, envid_t veid, const char *root,
 			int argc, char **argv);
+extern int console_attach(vps_handler *h, envid_t veid);
 
 static struct option set_opt[] = {
 	{"save",	no_argument, NULL, PARAM_SAVE},
@@ -137,6 +141,10 @@ static struct option set_opt[] = {
 	{"description",	required_argument, NULL, PARAM_DESCRIPTION},
 	{"bootorder",	required_argument, NULL, PARAM_BOOTORDER},
 
+	/* New "easy" VSwap parameters */
+	{"ram",		required_argument, NULL, PARAM_RAM},
+	{"swap",	required_argument, NULL, PARAM_SWAP},
+
 	{NULL, 0, NULL, 0}
 };
 
@@ -314,6 +322,10 @@ static int parse_create_opt(envid_t veid, int argc, char **argv,
 	{"hostname",	required_argument, NULL, PARAM_HOSTNAME},
 	{"name",	required_argument, NULL, PARAM_NAME},
 	{"description",	required_argument, NULL, PARAM_DESCRIPTION},
+	{"layout",	required_argument, NULL, PARAM_VE_LAYOUT},
+	{"ve_layout",	required_argument, NULL, PARAM_VE_LAYOUT},
+	{"velayout",	required_argument, NULL, PARAM_VE_LAYOUT},
+	{"diskspace",	required_argument, NULL, PARAM_DISKSPACE},
 	{ NULL, 0, NULL, 0 }
 };
 
@@ -341,15 +353,58 @@ static int destroy(vps_handler *h, envid_t veid, vps_param *g_p,
 {
 	int ret;
 
-	ret = vps_destroy(h, veid, &g_p->res.fs);
+	ret = vps_destroy(h, veid, &g_p->res.fs, &g_p->res.cpt);
 	if (!ret)
 		remove_names(veid);
 	return ret;
 }
 
+static int parse_convert_opt(envid_t veid, int argc, char **argv,
+	vps_param *param)
+{
+	int ret;
+	struct option *opt;
+	struct option convert_options[] = {
+	{"layout",	required_argument, NULL, PARAM_VE_LAYOUT},
+	{"ve_layout",	required_argument, NULL, PARAM_VE_LAYOUT},
+	{"velayout",	required_argument, NULL, PARAM_VE_LAYOUT},
+	{ NULL, 0, NULL, 0 }
+	};
+
+	opt = mod_make_opt(convert_options, &g_action, NULL);
+	if (opt == NULL)
+		return VZ_RESOURCE_ERROR;
+	ret = parse_opt(veid, argc, argv, opt, param);
+	free(opt);
+
+	return ret;
+}
+
+static int convert(vps_handler *h, envid_t veid, vps_param *g_p,
+		vps_param *cmd_p)
+{
+	int layout = cmd_p->opt.layout;
+	int mode = cmd_p->opt.mode;
+
+	/* Set defaults if not specified */
+	if (layout == 0)
+		layout = VE_LAYOUT_PLOOP;
+	if (mode < 0)
+		mode = PLOOP_EXPANDED_MODE;
+
+	/* We only support ploop here */
+	if (layout != VE_LAYOUT_PLOOP) {
+		logger(-1, 0, "Only conversion to ploop is supported");
+		return VZ_INVALID_PARAMETER_VALUE;
+	}
+
+	return vzctl_env_convert_ploop(h, veid,
+			&g_p->res.fs, &g_p->res.dq, mode);
+}
+
 static int parse_chkpnt_opt(int argc, char **argv, vps_param *vps_p)
 {
-	int c, ret;
+	int c;
 	int option_index;
 	cpt_param *cpt = &vps_p->res.cpt;
 	static struct option chkpnt_options[] = {
@@ -366,7 +421,6 @@ static int parse_chkpnt_opt(int argc, char **argv, vps_param *vps_p)
 	{ NULL, 0, NULL, 0 }
 	};
 
-	ret = 0;
 	while (1) {
 		option_index = -1;
 		c = getopt_long (argc, argv, "", chkpnt_options, NULL);
@@ -493,6 +547,50 @@ err_syntax:
 	return VZ_INVALID_PARAMETER_SYNTAX;
 }
 
+static int parse_snapshot_create_opt(envid_t veid, int argc, char **argv,
+		vps_param *param)
+{
+	int ret;
+	struct option *opt;
+	static struct option snapshot_create_options[] =
+	{
+	{"id",  required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{"uuid", required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{"name", required_argument, NULL, PARAM_SNAPSHOT_NAME},
+	{"description", required_argument, NULL, PARAM_SNAPSHOT_DESC},
+	{ NULL, 0, NULL, 0 }
+	};
+
+	opt = mod_make_opt(snapshot_create_options, &g_action, NULL);
+	if (opt == NULL)
+		return VZ_RESOURCE_ERROR;
+	ret = parse_opt(veid, argc, argv, opt, param);
+	free(opt);
+
+	return ret;
+}
+
+static int parse_snapshot_delete_opt(envid_t veid, int argc, char **argv,
+		vps_param *param)
+{
+	int ret;
+	struct option *opt;
+	static struct option snapshot_delete_options[] =
+	{
+	{"id",  required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{"uuid", required_argument, NULL, PARAM_SNAPSHOT_GUID},
+	{ NULL, 0, NULL, 0 }
+	};
+
+	opt = mod_make_opt(snapshot_delete_options, &g_action, NULL);
+	if (opt == NULL)
+		return VZ_RESOURCE_ERROR;
+	ret = parse_opt(veid, argc, argv, opt, param);
+	free(opt);
+
+	return ret;
+}
+
 static int check_set_ugidlimit(unsigned long *cur, unsigned long *old,
 		int loud)
 {
@@ -615,7 +713,7 @@ static int apply_param_from_cfg(int veid, vps_param *param, char *cfg)
 		return 0;
 	snprintf(conf, sizeof(conf), VPS_CONF_DIR "ve-%s.conf-sample", cfg);
 	if (!stat_file(conf)) {
-		logger(-1, 0, "Sample config file does not found: %s", conf);
+		logger(-1, 0, "Sample config file not found: %s", conf);
 		return VZ_APPLY_CONFIG_ERROR;
 	}
 	new = init_vps_param();
@@ -795,6 +893,14 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 			goto err;
 		}
 	}
+	/* Resize ploop image if --diskspace is supplied */
+	if (ve_private_is_ploop(g_p->res.fs.private) &&
+			cmd_p->res.dq.diskspace)
+	{
+		if ((ret = vzctl_resize_image(g_p->res.fs.private,
+						cmd_p->res.dq.diskspace[1])))
+			return ret;
+	}
 	/* Skip applying parameters on stopped CT */
 	if (cmd_p->opt.save && !is_run) {
 		ret = mod_setup(h, veid, STATE_STOPPED, SKIP_NONE, &g_action,
@@ -833,13 +939,12 @@ static int mount(vps_handler *h, envid_t veid, vps_param *g_p,
 static int umount(vps_handler *h, envid_t veid, vps_param *g_p,
 	vps_param *cmd_p)
 {
-	return vps_umount(h, veid, g_p->res.fs.root, 0);
+	return vps_umount(h, veid, &g_p->res.fs, 0);
 }
 
 static int enter(vps_handler *h, envid_t veid, const char *root,
 		int argc, char **argv)
 {
-	set_log_file(NULL);
 	if (check_var(root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (!vps_is_run(h, veid)) {
@@ -850,6 +955,8 @@ static int enter(vps_handler *h, envid_t veid, const char *root,
 		/* omit "--exec" argument */
 		argc -= 1; argv += 1;
 	}
+	logger(1, 0, "Entering CT");
+	set_log_file(NULL);
 	return do_enter(h, veid, root, argc, argv);
 }
 
@@ -863,30 +970,30 @@ static int exec(vps_handler *h, act_t action, envid_t veid, const char *root,
 	int i, ret;
 	int len, totallen = 0;
 
-	/* disable logging on terminal  */
-	set_log_verbose(0);
 	mode = MODE_BASH;
+	/* Unroll argv */
+	for (i = 0; i < argc; i++) {
+		len = strlen(argv[i]);
+		if (len) {
+			buf = (char*)realloc(buf, totallen + len + 2);
+			sprintf(buf + totallen, "%s ", argv[i]);
+		} else {
+			/* set empty argument */
+			len = 2;
+			buf = (char*)realloc(buf, totallen + len + 2);
+			sprintf(buf + totallen, "\'\' ");
+		}
+		totallen += len + 1;
+		buf[totallen] = 0;
+	}
 	if (action == ACTION_EXEC3) {
 		arg = argv;
 		mode = MODE_EXEC;
 	} else if (argc && strcmp(argv[0], "-")) {
-		for (i = 0; i < argc; i++) {
-			len = strlen(argv[i]);
-			if (len) {
-				buf = (char*)realloc(buf, totallen + len + 2);
-				sprintf(buf + totallen, "%s ", argv[i]);
-			} else {
-				/* set empty argument */
-				len = 2;
-				buf = (char*)realloc(buf, totallen + len + 2);
-				sprintf(buf + totallen, "\'\' ");
-			}
-			totallen += len + 1;
-			buf[totallen] = 0;
-		}
 		argv_bash[2] = buf;
 		arg = argv_bash;
 	}
+	logger(1, 0, "Executing command: %s", buf);
 	ret = vps_exec(h, veid, root, mode, arg, NULL, NULL, 0);
 
 	return ret;
@@ -894,13 +1001,36 @@ static int exec(vps_handler *h, act_t action, envid_t veid, const char *root,
 
 static int chkpnt(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *cmd_p)
 {
-	int cmd;
+	int cmd, ret;
 
 	cmd = cmd_p->res.cpt.cmd;
 	merge_vps_param(g_p, cmd_p);
-	if (cmd == CMD_KILL || cmd == CMD_RESUME)
-		return cpt_cmd(h, veid, CMD_CHKPNT, &cmd_p->res.cpt, g_p);
-	return vps_chkpnt(h, veid, g_p, cmd, &cmd_p->res.cpt);
+	cmd_p->res.cpt.dumpdir = g_p->res.cpt.dumpdir;
+	if (cmd == CMD_KILL || cmd == CMD_RESUME) {
+		ret = cpt_cmd(h, veid, g_p->res.fs.root,
+				CMD_CHKPNT, cmd, cmd_p->res.cpt.ctx);
+		if (ret)
+			return ret;
+		if (cmd == CMD_RESUME) {
+			/* restore arp/routing cleared on dump stage */
+			run_net_script(veid, ADD, &g_p->res.net.ip,
+				STATE_RUNNING,
+				cmd_p->res.net.skip_arpdetect);
+		}
+		return 0;
+	}
+
+	ret = vps_chkpnt(h, veid, &g_p->res.fs, cmd, &cmd_p->res.cpt);
+	if (ret)
+		return ret;
+	if (cmd == CMD_CHKPNT || cmd == CMD_DUMP) {
+		/* Clear CT network configuration */
+		run_net_script(veid, DEL, &g_p->res.net.ip, STATE_RUNNING,
+			cmd_p->res.net.skip_arpdetect);
+		if (cmd == CMD_CHKPNT)
+			vps_umount(h, veid, &g_p->res.fs, 0);
+	}
+	return 0;
 }
 
 static int restore(vps_handler *h, envid_t veid, vps_param *g_p,
@@ -911,7 +1041,8 @@ static int restore(vps_handler *h, envid_t veid, vps_param *g_p,
 	cmd = cmd_p->res.cpt.cmd;
 	merge_vps_param(g_p, cmd_p);
 	if (cmd == CMD_KILL || cmd == CMD_RESUME)
-		return cpt_cmd(h, veid, CMD_RESTORE, &cmd_p->res.cpt, g_p);
+		return cpt_cmd(h, veid, g_p->res.fs.root,
+				CMD_RESTORE, cmd, cmd_p->res.cpt.ctx);
 	return vps_restore(h, veid, g_p, cmd, &cmd_p->res.cpt);
 }
 
@@ -966,6 +1097,9 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	case ACTION_CREATE:
 		ret = parse_create_opt(veid, argc, argv, param);
 		break;
+	case ACTION_CONVERT:
+		ret = parse_convert_opt(veid, argc, argv, param);
+		break;
 	case ACTION_START:
 		ret = parse_startstop_opt(argc, argv, param, 1, 0);
 		break;
@@ -1018,6 +1152,17 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	case ACTION_RESTORE:
 		ret = parse_restore_opt(argc, argv, param);
 		break;
+	case ACTION_CONSOLE:
+		break;
+	case ACTION_SNAPSHOT_CREATE:
+		ret = parse_snapshot_create_opt(veid, argc, argv, param);
+		break;
+	case ACTION_SNAPSHOT_DELETE:
+	case ACTION_SNAPSHOT_SWITCH:
+		ret = parse_snapshot_delete_opt(veid, argc, argv, param);
+		break;
+	case ACTION_SNAPSHOT_LIST:
+		break;
 	default :
 		if ((argc - 1) > 0) {
 			fprintf (stderr, "Invalid options: ");
@@ -1053,6 +1198,7 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		action != ACTION_EXEC2 &&
 		action != ACTION_EXEC3 &&
 		action != ACTION_ENTER &&
+		action != ACTION_CONSOLE &&
 		action != ACTION_STATUS)
 	{
 		if (skiplock != YES) {
@@ -1094,6 +1240,9 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 	case ACTION_RESTART:
 		ret = restart(h, veid, g_p, cmd_p);
 		break;
+	case ACTION_CONVERT:
+		ret = convert(h, veid, g_p, cmd_p);
+		break;
 	case ACTION_SET:
 		if (veid == 0)
 			ret = set_ve0(h, g_p, vps_p, cmd_p);
@@ -1109,9 +1258,11 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 					cmd_p, vps_p, &g_action);
 		} else if (cmd_p->opt.save != NO) {
 			if (list_empty(&cmd_p->res.misc.userpw)) {
+				int is_run = h != NULL && vps_is_run(h, veid);
 				logger(0, 0, "WARNING: Settings were not saved"
-				" and will be reset to original values at the"
-				" next start (use --save flag)");
+				" to config %s(use --save flag)",
+				is_run ? "and will be lost after CT restart "
+					: "");
 			}
 		}
 		break;
@@ -1121,6 +1272,9 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 	case ACTION_ENTER:
 		ret = enter(h, veid, g_p->res.fs.root, argc, argv);
 		break;
+	case ACTION_CONSOLE:
+		ret = console_attach(h, veid);
+		break;
 	case ACTION_EXEC:
 	case ACTION_EXEC2:
 	case ACTION_EXEC3:
@@ -1138,11 +1292,19 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		ret = vps_run_script(h, veid, argv[0], g_p);
 		break;
 
-#define CHECK_DQ(r) if (g_p->res.dq.enable != YES) { \
-	ret = r; \
-	logger(0, 0, "Disk quota is not enabled, skipping operation"); \
-	break; \
-}
+#define CHECK_DQ(r)								\
+	if (g_p->res.dq.enable != YES) {					\
+		ret = r;							\
+		logger(0, 0, "Disk quota is not enabled, skipping operation");	\
+		break;								\
+	}									\
+	if (ve_private_is_ploop(g_p->res.fs.private)) {				\
+		ret = 0;							\
+		logger(0, 0, "CT is on ploop, disk quota is obsoleted, "	\
+				"skipping operation");				\
+		break;								\
+	}
+
 	case ACTION_QUOTAON:
 		CHECK_DQ(VZ_DQ_ON)
 		ret = vps_quotaon(veid, g_p->res.fs.private, &g_p->res.dq);
@@ -1156,7 +1318,21 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		ret = quota_init(veid, g_p->res.fs.private, &g_p->res.dq);
 		break;
 #undef CHECK_DQ
-
+	case ACTION_SNAPSHOT_CREATE:
+		ret = vzctl_env_create_snapshot(h, veid,
+				&g_p->res.fs, &cmd_p->snap);
+		break;
+	case ACTION_SNAPSHOT_DELETE:
+		ret = vzctl_env_delete_snapshot(h, veid,
+				&g_p->res.fs, cmd_p->snap.guid);
+		break;
+	case ACTION_SNAPSHOT_SWITCH:
+		ret = vzctl_env_switch_snapshot(h, veid, g_p, &g_p->res.fs,
+				cmd_p->snap.guid);
+		break;
+	case ACTION_SNAPSHOT_LIST:
+		ret = vzctl_env_list_snapshot_tree(g_p->res.fs.private);
+		break;
 	case ACTION_CUSTOM:
 		ret = mod_setup(h, veid, 0, 0, &g_action, g_p);
 		break;
