@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2012, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -205,6 +205,16 @@ static void print_bootorder(struct Cveinfo *p, int index)
 				"%10lu", p->bootorder[index]);
 }
 
+static void print_cpunum(struct Cveinfo *p, int index)
+{
+	if (p->cpunum <= 0)
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%5s", "-");
+	else
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%5d", p->cpunum);
+}
+
 #define PRINT_UBC(name)							\
 static void print_ubc_ ## name(struct Cveinfo *p, int index)		\
 {									\
@@ -345,6 +355,12 @@ static int ioprio_sort_fn(const void *val1, const void *val2)
 		((const struct Cveinfo *)val2)->io.ioprio;
 }
 
+static int cpunum_sort_fn(const void *val1, const void *val2)
+{
+	return ((const struct Cveinfo *)val1)->cpunum >
+		((const struct Cveinfo *)val2)->cpunum;
+}
+
 #define SORT_STR_FN(name)						\
 static int name ## _sort_fn(const void *val1, const void *val2)		\
 {									\
@@ -471,6 +487,7 @@ UBC_FIELD(swappages, SWAPP),
 
 {"cpulimit", "CPULIM", "%7s", 0, RES_CPU, print_cpulimit, cpulimit_sort_fn},
 {"cpuunits", "CPUUNI", "%7s", 1, RES_CPU, print_cpulimit, cpuunits_sort_fn},
+{"cpus", "CPUS", "%5s", 0, RES_CPUNUM, print_cpunum, cpunum_sort_fn},
 
 {"ioprio", "IOP", "%3s", 0, RES_NONE, print_ioprio, ioprio_sort_fn},
 
@@ -483,7 +500,7 @@ static void *x_malloc(int size)
 {
 	void *p;
 	if ((p = malloc(size)) == NULL) {
-		printf("Error: unable to allocate %d bytes\n", size);
+		fprintf(stderr, "Error: unable to allocate %d bytes\n", size);
 		exit(1);
 	}
 	return p;
@@ -494,7 +511,7 @@ static void *x_realloc(void *ptr, int size)
 	void *tmp;
 
 	if ((tmp = realloc(ptr, size)) == NULL) {
-		printf("Error: unable to allocate %d bytes\n", size);
+		fprintf(stderr, "Error: unable to allocate %d bytes\n", size);
 		exit(1);
 	}
 	return tmp;
@@ -502,7 +519,7 @@ static void *x_realloc(void *ptr, int size)
 
 static void usage()
 {
-	fprintf(stderr,
+	printf(
 "Usage:	vzlist [-a | -S] [-n] [-H] [-o field[,field...] | -1] [-s [-]field]\n"
 "	       [-h pattern] [-N pattern] [-d pattern] [CTID [CTID ...]]\n"
 "	vzlist -L | --list\n"
@@ -653,7 +670,7 @@ static void add_elem(struct Cveinfo *ve)
 	return;
 }
 
-static inline struct Cveinfo *find_ve(int veid)
+static struct Cveinfo *find_ve(int veid)
 {
 	return (struct Cveinfo *) bsearch(&veid, veinfo, n_veinfo,
 			sizeof(struct Cveinfo), id_search_fn);
@@ -716,17 +733,6 @@ static void update_cpu(int veid, unsigned long limit, unsigned long units)
 	return;
 }
 
-static void update_cpustat(int veid, struct Ccpustat *st)
-{
-	struct Cveinfo *tmp;
-
-	if ((tmp = find_ve(veid)) == NULL)
-		return;
-	tmp->cpustat = x_malloc(sizeof(*st));
-	memcpy(tmp->cpustat, st, sizeof(*st));
-	return;
-}
-
 #define MERGE_QUOTA(name, quota, dq)				\
 do {								\
 	if (dq.name != NULL) {					\
@@ -816,6 +822,8 @@ do {								\
 		*ve->bootorder = *res->misc.bootorder;
 	}
 	ve->io.ioprio = res->io.ioprio;
+	if (res->cpu.vcpus != NULL)
+		ve->cpunum = *res->cpu.vcpus;
 }
 
 static int read_ves_param()
@@ -888,7 +896,8 @@ static int get_ub()
 
 	if ((fp = fopen(PROC_BC_RES, "r")) == NULL) {
 		if ((fp = fopen(PROCUBC, "r")) == NULL) {
-			fprintf(stderr, "Unable to open %s\n", PROCUBC);
+			fprintf(stderr, "Unable to open %s: %s\n",
+					PROCUBC, strerror(errno));
 			return 1;
 		}
 	}
@@ -1014,7 +1023,8 @@ static int get_run_ve_proc(int update)
 	int res, veid, classid, nproc;
 
 	if ((fp = fopen(PROCVEINFO, "r")) == NULL) {
-		fprintf(stderr, "Unable to open %s\n", PROCVEINFO);
+		fprintf(stderr, "Unable to open %s: %s\n",
+				PROCVEINFO, strerror(errno));
 		return 1;
 	}
 	memset(&ve, 0, sizeof(struct Cveinfo));
@@ -1192,13 +1202,13 @@ static long get_clk_tck()
 	return __clk_tck;
 }
 
-static int get_ve_cpustat(int veid)
+static int get_ve_cpustat(struct Cveinfo *ve)
 {
 	struct vz_cpu_stat stat;
 	struct vzctl_cpustatctl statctl;
 	struct Ccpustat st;
 
-	statctl.veid = veid;
+	statctl.veid = ve->veid;
 	statctl.cpustat = &stat;
 	if (ioctl(vzctlfd, VZCTL_GET_CPU_STAT, &statctl) != 0)
 		return 1;
@@ -1208,7 +1218,8 @@ static int get_ve_cpustat(int veid)
 
 	st.uptime = (float) stat.uptime_jif / get_clk_tck();
 
-	update_cpustat(veid, &st);
+	ve->cpustat = x_malloc(sizeof(st));
+	memcpy(ve->cpustat, &st, sizeof(st));
 	return 0;
 }
 
@@ -1221,7 +1232,7 @@ static int get_ves_cpustat()
 	for (i = 0; i < n_veinfo; i++) {
 		if (veinfo[i].hide)
 			continue;
-		get_ve_cpustat(veinfo[i].veid);
+		get_ve_cpustat(&veinfo[i]);
 	}
 	close(vzctlfd);
 	return 0;
@@ -1252,6 +1263,42 @@ static int get_mounted_status()
 	return 0;
 }
 
+static int get_ve_cpunum(struct Cveinfo *ve) {
+	char path[] = "/proc/vz/fairsched/2147483647/cpu.nr_cpus";
+	int veid = ve->veid;
+	FILE *fp;
+	int ret = -1;
+
+	snprintf(path, sizeof(path),
+			"/proc/vz/fairsched/%d/cpu.nr_cpus", veid);
+	fp = fopen(path, "r");
+	if (fp == NULL) {
+		fprintf(stderr, "Unable to open %s: %s\n",
+				path, strerror(errno));
+		return -1;
+	}
+	if (fscanf(fp, "%d", &ve->cpunum) != 1)
+		goto out;
+
+	ret = 0;
+out:
+	fclose(fp);
+
+	return ret;
+}
+
+static int get_ves_cpunum()
+{
+	int i;
+
+	for (i = 0; i < n_veinfo; i++) {
+		if ((veinfo[i].hide) || (veinfo[i].status != VE_RUNNING))
+			continue;
+		get_ve_cpunum(&veinfo[i]);
+	}
+	return 0;
+}
+
 static int get_ves_cpu()
 {
 	unsigned long tmp;
@@ -1260,7 +1307,8 @@ static int get_ves_cpu()
 	char buf[128];
 
 	if ((fp = fopen(PROCFSHED, "r")) == NULL) {
-		fprintf(stderr, "Unable to open %s\n", PROCFSHED);
+		fprintf(stderr, "Unable to open %s: %s\n",
+				PROCFSHED, strerror(errno));
 		return 1;
 	}
 	veid = 0;
@@ -1397,6 +1445,9 @@ static int collect()
 	if (check_param(RES_CPU))
 		if (!only_stopped_ve && (ret = get_ves_cpu()))
 			return ret;
+	if (check_param(RES_CPUNUM))
+		if (!only_stopped_ve && (ret = get_ves_cpunum()))
+			return ret;
 	read_ves_param();
 	get_mounted_status();
 	if (host_pattern != NULL)
@@ -1509,7 +1560,8 @@ int main(int argc, char **argv)
 				sort_rev = 1;
 			}
 			if ((g_sort_field = search_field(p)) < 0) {
-				printf("Invalid sort field name: %s\n", optarg);
+				fprintf(stderr, "Invalid sort field name: "
+						"%s\n", optarg);
 				return 1;
 			}
 			break;
@@ -1525,6 +1577,7 @@ int main(int argc, char **argv)
 			name_pattern = strdup(optarg);
 			break;
 		default		:
+			/* "Unknown option" error msg is printed by getopt */
 			usage();
 			return 1;
 		}
@@ -1555,8 +1608,15 @@ int main(int argc, char **argv)
 		fprintf(stderr, "This program can only be run under root.\n");
 		return 1;
 	}
-	if ((ret = collect()))
-		return ret;
+	if ((ret = collect())) {
+		/* If no specific CTIDs are specified in arguments,
+		 * 'no containers found' is not an error (bug #2149)
+		 */
+		if (g_ve_list == NULL)
+			return 0;
+		else
+			return ret;
+	}
 	print_ve();
 	free_veinfo();
 	free(host_pattern);
